import { __decorate } from "tslib";
import { join } from 'node:path';
import { handle, wordpress } from '@roots/wordpress-transforms';
import { bind } from 'helpful-decorators';
import Webpack from 'webpack';
export default class WordPressDependenciesWebpackPlugin {
    options;
    dependencies;
    outputPath;
    plugin = {
        name: `WordPressDependenciesWebpackPlugin`,
        stage: Infinity,
    };
    requested;
    constructor(options = { emitWordPressJson: true }) {
        this.options = options;
        this.outputPath = this.options.outputPath ?? `wordpress.json`;
        if (typeof this.options.emitWordPressJson === `undefined` &&
            !this.options.entrypointsPlugin)
            this.options.emitWordPressJson = true;
        this.dependencies = new Map();
        this.requested = new Map();
    }
    /**
     * Add item to set in map
     *
     * @remarks
     * Works for both our map of requests and our map of dependencies.
     */
    addItemToMap(obj, key, item) {
        const items = obj.get(key);
        if (!items) {
            obj.set(key, new Set([item]));
            return;
        }
        obj.set(key, items.add(item));
    }
    /**
     * Apply plugin
     */
    apply(compiler) {
        compiler.hooks.normalModuleFactory.tap(this.plugin.name, factory => {
            factory.hooks.beforeResolve.tap(this.plugin.name, ({ contextInfo, request }) => {
                this.addItemToMap(this.requested, contextInfo.issuer, request);
            });
        });
        compiler.hooks.thisCompilation.tap(this.plugin, compilation => {
            if (!this.options)
                return;
            if (this.options.entrypointsPlugin) {
                const hooks = this.options.entrypointsPlugin.getCompilationHooks(compilation);
                hooks.compilation.tap(this.plugin.name, this.extractDependenciesFromCompilation);
                hooks.entrypoints.tap(this.plugin.name, this.tapEntrypointsManifestObject);
            }
            if (this.options?.emitWordPressJson) {
                compilation.hooks.processAssets.tapPromise(this.plugin, async () => {
                    this.extractDependenciesFromCompilation(compilation);
                    compilation.emitAsset(this.outputPath, new compiler.webpack.sources.RawSource(JSON.stringify(this.dependencies, null, 2)));
                });
            }
        });
    }
    extractDependenciesFromCompilation(compilation) {
        for (const { chunks, name } of compilation.entrypoints.values()) {
            if (!name)
                continue;
            for (const chunk of chunks) {
                const records = compilation.chunkGraph.getChunkModules(chunk);
                for (const { userRequest } of records) {
                    this.processChunkRequest(name, userRequest);
                }
                for (const { modules } of records) {
                    if (!modules)
                        continue;
                    for (const { userRequest } of modules) {
                        this.processChunkRequest(name, userRequest);
                    }
                }
            }
        }
    }
    /**
     * Add mapped request to entrypoint manifest object
     */
    processChunkRequest(name, userRequest) {
        if (!name || !userRequest)
            return;
        const requested = this.requested.get(userRequest);
        if (!requested)
            return;
        for (const request of requested) {
            if (this.options.exclude?.includes(request))
                continue;
            /**
             * It's harder to exclude react-refresh-runtime
             * because it is often transitively included.
             *
             * So we check if the request includes the string
             * and if it does, we check if it is in the exclude
             * array.
             *
             * It is a bit of a hack, but it works.
             */
            if (this.options?.exclude?.includes(join(`react-refresh`, `runtime`)) &&
                request.includes(join(`react-refresh`, `runtime`))) {
                continue;
            }
            if (!wordpress.isProvided(request))
                continue;
            const wordPressHandle = handle.transform(request);
            if (!wordPressHandle)
                continue;
            this.addItemToMap(this.dependencies, name, wordPressHandle);
        }
    }
    /**
     * Tap entrypoints manifest object
     */
    tapEntrypointsManifestObject(entrypoints) {
        for (const [ident, entrypoint] of entrypoints.entries()) {
            const dependencies = this.dependencies.get(ident);
            if (!dependencies) {
                entrypoint.set(`dependencies`, new Set());
                continue;
            }
            for (const dependency of dependencies) {
                const entrypointDependencies = entrypoint.get(`dependencies`);
                if (!entrypointDependencies) {
                    entrypoint.set(`dependencies`, new Set([dependency]));
                    continue;
                }
                entrypointDependencies.add(dependency);
            }
        }
        return entrypoints;
    }
}
__decorate([
    bind
], WordPressDependenciesWebpackPlugin.prototype, "apply", null);
__decorate([
    bind
], WordPressDependenciesWebpackPlugin.prototype, "extractDependenciesFromCompilation", null);
__decorate([
    bind
], WordPressDependenciesWebpackPlugin.prototype, "processChunkRequest", null);
__decorate([
    bind
], WordPressDependenciesWebpackPlugin.prototype, "tapEntrypointsManifestObject", null);
